<?php

namespace Drupal\Tests\consumers\Functional;

use Drupal\Component\Serialization\Json;
use Drupal\consumers\Entity\Consumer;
use Drupal\Tests\jsonapi\Functional\JsonApiFunctionalTestBase;

/**
 * Tests the cacheability of the consumer client id field.
 *
 * @group consumers
 */
class CacheabilityTest extends JsonApiFunctionalTestBase {

  /**
   * The required modules to install.
   *
   * @var array
   */
  protected static $modules = [
    'consumers',
    'consumers_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The content type for testing.
   *
   * @var \Drupal\node\Entity\NodeType
   */
  protected $contentType;

  /**
   * Test node.
   *
   * @var \Drupal\node\Entity\Node
   */
  protected $node;

  /**
   * Test consumer.
   *
   * @var \Drupal\consumers\Entity\Consumer
   */
  protected $consumer;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->contentType = $this->drupalCreateContentType([
      'type' => CONSUMERS_TEST_NODE_TYPE,
    ]);
    $this->node = $this->createNode([
      'type' => CONSUMERS_TEST_NODE_TYPE,
    ]);
    $this->consumer = Consumer::create([
      'client_id' => $this->randomMachineName(),
      'label' => $this->randomString(),
    ]);
    $this->consumer->save();
    drupal_flush_all_caches();
  }

  /**
   * Check client id cacheability depending on provided consumer.
   */
  public function testClientIdCacheability() {
    $path = sprintf(
      '/jsonapi/node/%s/%s',
      $this->contentType->id(),
      $this->node->uuid()
    );
    $query = [
      'consumerId' => $this->consumer->getClientId(),
    ];

    // Pass test consumer and make sure its client id is returned.
    $output = Json::decode($this->drupalGet($path, ['query' => $query]));
    $this->assertArrayHasKey('consumer_client_id', $output['data']['attributes']);
    $this->assertEquals($this->consumer->getClientId(), $output['data']['attributes']['consumer_client_id']);

    // Check if test consumer result was not cached and default returned
    // when no consumer passed.
    $output = Json::decode($this->drupalGet($path));
    $this->assertArrayHasKey('consumer_client_id', $output['data']['attributes']);
    $this->assertEquals('default_consumer', $output['data']['attributes']['consumer_client_id']);
  }

}
